function [phi, resid, rssq] =  setar41estim(y, ttau, d, p)
%    setar41estim estimates a non stationary autoregressive model with a time break   
%    y= time series data, **  column vector  **
%    ttau =  time breaks locations, including interval extremes (0,n)
%    d  =  delay parameter (dummy parameter)
%    p  =  autoregressive order
% --- notice ttau: ttau(1)= 0 (value less than the first observation), 
%                ttau(nt+1)=n (the program check for <=)
%    OUTPUT parameter phi each row is an AR parameter sequence, rows are
%    ordered time break regime (linear models for each time interval)
%    RESIDUALS in output are a column vector
%    rssq  is the RESIDUAL SUM OF SQUARES, not the residual variance
n = size(y, 1);
nt1 = size(ttau, 2); nt = nt1-1; %    nt = number of regimes  
m =  max(p, d); % observations lost at the beginning of the time series
Z = y(m+1:n); % dependent variable
ZS = m+1:1:n; %  time
ZMAT = ones(n-m, p+1); % initialize the indpendent variables
phi =  zeros(nt, p+1);  %  initialize the parameter array
for ic = 1:p, ZMAT(:,ic+1) = y(m-ic+1:n-ic); end, %ZMAT, % compute the independet var
for  regime = 1:nt, % for each regime performs least squares estimation
    clear r1,  clear r2, r1 = ttau(regime);  r2 = ttau(regime+1);  %  r2=threshold
    clear idx, idx  =  find(ZS>r1 & ZS<=r2); % index of the observations in the regime
    clear X, clear XMAT, X = Z(idx); XMAT= ZMAT(idx,:); %  prepare for regression
    phi(regime,:) = (XMAT\X)';  % lesast squraes solution of equation XMAT*phi=X
end,
%phi, 
resid = zeros(n,1); % initialize the residual column vector
for  regime = 1:nt, % for each regime compute the residuals
    clear r1,  clear r2, r1 = ttau(regime);  r2 = ttau(regime+1);  %  r2=threshold
    clear idx, idx  =  find(ZS>r1 & ZS<=r2); %qqx=idx+m,% index of the observations in the regime
    clear X, clear XMAT, X = Z(idx); XMAT= ZMAT(idx,:); %  prepare for regression
    clear th, th=(phi(regime,:))'; resid(idx+m)=X-XMAT*th;  % residuals in regime with y*=XMAT*th
end,
%rssq = norm(resid), %  norm computes the square root of the sum of squares 
rssq  =  resid'*resid;   %  the residual sum of squares 
%   setar41estim  program estimate ends