%  star4estim
%  estimate of a non-stationary state-variable-varying STAR model 
%  input time series y and output residuals u are column vectors
%  rssq is the residuals sum-of-squares 
function [phi, u, rssq] =  star4estim(y, d, p, v1, v2),
%  y = time series, column vector
%  d = delay parameter
%  p = autoregressive order
%  v1(1) = threshold  in the logistic function /state variable
%  v1(2)  = gamma parameter in the logistic function /state variable
%  v2(1) = threshold  in the logistic function /time break
%  v2(2)  = gamma parameter in the logistic function /time break
n  = size(y, 1);
m = max(p,d);
Z = y(m+1:n); % dependent variable
ZS = y(m+1-d:n-d);  G1 = gstar(ZS,v1); % state variable and logistic factor
tti=  (m+1:1:n)';   G2= gstar(tti, v2); % logistic coef. as a function of time 
ZMAT = zeros(n-m,p+1); %  prepares the independent variables
yp =  0;  %  parameter counter, each term has 2 parameters
yp =  yp+1;  ZMAT(:,yp)= ones(n-m,1);  %  the constant term (does not change)
yp =  yp+1;  ZMAT(:,yp) = G1; % constant term, varies with the state var.
yp =  yp+1;  ZMAT(:,yp) = G2; % constant term, varies with time
if p>0, % autoregressive terms, if present
    for ji = 1:p,
        yp=yp+1; ZMAT(:,yp) = y(m+1-ji:n-ji); %lagged variable 
        yp=yp+1; ZMAT(:,yp) = G1.*y(m+1-ji:n-ji); %lagged var.* state var.
        yp=yp+1; ZMAT(:,yp) = G2.*y(m+1-ji:n-ji); %lagged var.* time break
    end,
end,
% Z, ZMAT, 
phi =  ZMAT\Z;  % lesast squraes solution of equation ZMAT*phi=Z
u = [zeros(m,1); (Z- ZMAT*phi)];   %  compute residuals as a column vector 
rssq = u'*u;    %  residuals  sum of squares


%  [FINISH]  star4estim
