function [phi, resid, rssq] =  tvymr4estim(y, d, p, mmask, thres, ttau, type2),
%  -- time varying multi-regime model paramters estimation least squares routine
%    y=    input time series data, *column* vector
%    d  =  delay parameter
%    p  =  autoregressive order
%    mmask =  in the 9-terms equation of tvymr model specifies the zero terms
%    thres =  threshold values, including interval extremes (row vector)
%    ttau  =  time breaks, first element 0 last element n (row vector)
%    type2 =  row vector 2 entries, first=model state, second=model break
%             1 = SETAR : 2 = PLTAR : 3 = STAR
% --- notice thres: thres(1)=value less than the smallest observation, 
%       thres(nr+1)=largest observation (the program check for <=)
% -- otput:
%    phi   = estimated parameters, column vector
%    resid = estimated residuals, *column* vector
%    rssq  = residual sum of squares
n = size(y, 1);   
nr1  = size(thres, 2); nr = nr1-1; %    nr = number of regimes in state levels
  nt1 =  size(ttau,  2); nt = nt1-1;  %  nt = number of regimes in time
m =  max(p, d); % observations lost at the beginning of the time series
Z = y(m+1:n); % dependent variable
ZS = y(m-d+1:n-d); %  state variable
ti=(m+1:1:n)';  %  time
  %   size(ZS,1), size(ti,1),
  %   compute the number of parameters nparam
 tparam = [1+p  1+p  (1+p)*nt  1+p  1+p  (1+p)*nt  (1+p)*nr  (1+p)*nr  (1+p)*nt*nr];
  nparam = mmask*tparam';
ZMAT = ones(n-m, nparam); % initialize the indpendent variables
phi =  zeros(nparam, 1);  %  initialize the parameter array
  qxmat=qxfunct(type2, ZS, thres, ti, ttau); % 0-1 matrix of regimes intersection
  qhstate =  qhfunct(type2(1), ZS, thres);
  qhbreak =  qhfunct(type2(2), ti, ttau);
  yp = 0;  %  the constant term comes first
        if mmask(1) == 1,
            yp = yp + 1;
            ZMAT(:,yp) =  ones(n -m, 1);
        end,
        if  mmask(2) == 1, 
            yp = yp + 1;
            ZMAT(:,yp) =  ti;
        end,
        if  mmask(3) == 1,
            yp = yp + 1; %qhfunct gives nt columns of 0 and 1
            ZMAT(:,yp:yp+nt-1) =  qhbreak;
             yp = yp+nt -1;
       end,
        if  mmask(4) == 1, 
            yp = yp + 1;
            ZMAT(:,yp) =  ZS;
        end,
        if  mmask(5) == 1, 
            yp = yp + 1;
            ZMAT(:,yp) =  ti.*ZS;
        end,
        if  mmask(6) == 1, 
            yp = yp + 1;
            ZMAT(:,yp:yp+nt-1) =  repmat(ZS, 1, nt).*qhbreak;
            yp = yp+nt -1;
        end,
        if  mmask(7) == 1, 
            yp = yp + 1;
            ZMAT(:,yp:yp+nr-1) =  qhstate;
            yp = yp+nr -1;
        end,
        if  mmask(8) == 1, 
            yp = yp + 1;
            ZMAT(:,yp:yp+nr-1) =  repmat(ti, 1, nr).*qhstate;
            yp = yp+nr -1;
        end,
        if  mmask(9) == 1, 
            yp = yp + 1;
            ZMAT(:,yp:yp+nr*nt-1) =  qxmat;
            yp = yp+nr*nt-1;
        end,
  %  then the lagged terms come   
   for jj = 1:p, 
        if mmask(1) == 1,
            yp = yp + 1;
            ZMAT(:,yp) =  y(m+1-jj:n-jj);
        end,
        if  mmask(2) == 1, 
            yp = yp + 1;
            ZMAT(:,yp) =  ti.*y(m+1-jj:n-jj);
        end,
        if  mmask(3) == 1,
            yp = yp + 1; %qhfunct gives nt columns of 0 and 1
            ZMAT(:,yp:yp+nt-1) =  qhbreak.*repmat(y(m+1-jj:n-jj),1,nt);
             yp = yp+nt -1;
       end,
        if  mmask(4) == 1, 
            yp = yp + 1;
            ZMAT(:,yp) =  ZS.*y(m+1-jj:n-jj);
        end,
        if  mmask(5) == 1, 
            yp = yp + 1;
            ZMAT(:,yp) =  ti.*ZS.*y(m+1-jj:n-jj);
        end,
        if  mmask(6) == 1, 
            yp = yp + 1;
            ZMAT(:,yp:yp+nt-1) =  repmat(ZS,1,nt).*qhbreak.*repmat(y(m+1-jj:n-jj),1,nt);
            yp = yp+nt -1;
        end,
        if  mmask(7) == 1, 
            yp = yp + 1;
            ZMAT(:,yp:yp+nr-1) =  qhstate.*repmat(y(m+1-jj:n-jj),1,nr);
            yp = yp+nr -1;
        end,
        if  mmask(8) == 1, 
            yp = yp + 1;
            ZMAT(:,yp:yp+nr-1) =  repmat(ti, 1, nr).*qhstate.*repmat(y(m+1-jj:n-jj),1,nr);
            yp = yp+nr -1;
        end,
        if  mmask(9) == 1, 
            yp = yp + 1; % qxfunct gives nr*nt columns of 0 and 1
            ZMAT(:,yp:yp+nr*nt-1) = qxmat .* repmat(y(m+1-jj:n-jj), 1, nr*nt);
            yp = yp+nr*nt-1;
        end,
    end,
     %size_Z=size(Z), size_ZMAT=size(ZMAT), 
    phi  = (ZMAT\Z);  % lesast squraes solution of equation ZMAT*phi=Z
     %size_phi=size(phi),
  resid(1:m, 1)= zeros(m,1); resid(m+1:n,1)= Z- ZMAT*phi;  %  residuals
     %size_resid=size(resid),
    rssq = resid'*resid; %   the residual sum of squares 

%   tvymr4estim  (fin)